/*subprograms needed to fit a gaussian to an array of points*/
/*PASSED TEST W/ FLYING COLORS*/
/*The code was written and debugged by John C. Martin at the University of
  Minnesota (copyright 2004,2005).  You are free to distribute this source
  as long as this notice remains in tact.  End users of this code agree to
  take responsibility for how this code is used and John C. Martin and the
  University of Minnesota bear no liability.*/

#include "mkvector.h"
#include <stdio.h>
#include <math.h>
void gaussjordan(double **a, int nc, double *b);

void fitgaus(double *y, int ndata, double *a)
     /*y is the array of equally weighted y values where the array index is x
       ndata is the number of y values
       a [1..3] is the parameters in the order (amplitude, center, hwhm)
           a should contain an inital guess at the parameters
     This function will solve all three parameters simultaneously*/
     /*  Based on the Levenberg-Marquardt method*/
{
  double x[ndata+1];
  int i,j,k,l,m,debug;
  double **cv,**al,alm,lalm,chs,ochs,atry[4],da[4],bet[4];
  double dyda[4],wt,yf,arg,ex,fac,dy;
  float fgauss(float,float [],float *,float [], int);

  debug=0;
  for (i=1;i<=ndata;i++) {x[i]=(float)i-1;}
  al=dvector2d(1,3,1,3);
  cv=dvector2d(1,3,1,3);
  alm=0.001;

  if (debug) fprintf(stderr,"Initial guess: %e %e %e\n",a[1],a[2],a[3]);
  /*Init Matrices*/
  for (j=1;j<=3;j++){
    for (k=1;k<=j; k++) al[j][k]=0.0;
    bet[j]=0.0;
  }
  chs=0.0;

  /*Init Loop*/
  for (i=1;i<=ndata;i++) {
    /*Gaussian Function and derivatives*/
    arg=(x[i]-a[2])/a[3];
    ex=exp(-arg*arg);
    fac=a[1]*ex*2.0*arg;
    yf=a[1]*ex;
    dyda[1]=ex;
    dyda[2]=fac/a[3];
    dyda[3]=fac*arg/a[3];

    dy=y[i]-yf;
    for (j=0,l=1;l<=3;l++) {
      wt=dyda[l];
      for (j++,k=0,m=1;m<=l;m++)
	al[j][++k] += wt*dyda[m];
      bet[j] += dy*wt;
    }
    /*Noisy debuging code
    if (debug) fprintf(stderr,"  [%e %e %e]      [%e]       [%e]\n",
		       al[1][1],al[1][2],al[1][3],bet[1],dyda[1]);
    if (debug) fprintf(stderr,"a=[%e %e %e]    b=[%e]  dyda=[%e]\n",
		       al[2][1],al[2][2],al[2][3],bet[2],dyda[2]);
    if (debug) fprintf(stderr,"  [%e %e %e]      [%e]       [%e]\n\n",
		       al[3][1],al[3][2],al[3][3],bet[3],dyda[3]);
    */
    chs += dy*dy;
  }
  for (j=2;j<=3;j++)
    for (k=1;k<j;k++) al[k][j]=al[j][k];

  ochs=chs;
  for (j=1;j<=3;j++) atry[j]=a[j];
  lalm=alm;

  /*MAIN LOOP*/
  while (lalm>=alm) {
    lalm=alm;
    for (j=0,l=1;l<=3;l++){
      for (j++,k=0,m=1;m<=3;m++) {
	  k++;
	  cv[j][k]=al[j][k];
      }
      cv[j][j]=al[j][j]*(1.0+(alm));
    }

    gaussjordan(cv,3,bet);

    for (j=1;j<=3;j++) da[j]=bet[j];
    for (j=0,l=1;l<=3;l++) atry[l]=a[l]+da[++j];

    //==========================================================
    for (j=1;j<=3;j++){
      for (k=1;k<=j; k++) cv[j][k]=0.0;
      da[j]=0.0;
    }
    chs=0.0;
    for (i=1;i<=ndata;i++) {
      /*Gaussian Function and derivatives*/
      arg=(x[i]-atry[2])/atry[3];
      ex=exp(-arg*arg);
      fac=atry[1]*ex*2.0*arg;
      yf=atry[1]*ex;
      dyda[1]=ex;
      dyda[2]=fac/atry[3];
      dyda[3]=fac*arg/atry[3];

      dy=y[i]-yf;
      for (j=0,l=1;l<=3;l++) {
	wt=dyda[l];
	for (j++,k=0,m=1;m<=l;m++)
	  cv[j][++k] += wt*dyda[m];
	da[j] += dy*wt;
      }
      chs += dy*dy;
    }
    for (j=2;j<=3;j++)
      for (k=1;k<j;k++) cv[k][j]=cv[j][k];
    //==============================================================

    if (debug) fprintf(stderr,"chisqr= %e  old_chisqr= %e\n",chs,ochs);
    if (debug) fprintf(stderr,"atry = [%e,%e,%e]\n",atry[1],atry[2],atry[3]);
    if (chs < ochs) {
      alm *= 0.1;
      ochs=(chs);
      for (j=0,l=1;l<=3;l++) {
      	for (j++,k=0,m=1;m<=3;m++) {
	    k++;
	    al[j][k]=cv[j][k];
	}
	bet[j]=da[j];
	a[l]=atry[l];
      }
    } else { 
      alm *=10.0;
      chs=ochs;
    }
    if (debug) fprintf(stderr,"Alam=%e\n",alm);
  }
  if (debug) fprintf(stderr,"Results: %e %e %e\n\n",a[1],a[2],a[3]);

}
