/*The code was written and debugged by John C. Martin at the University of
  Minnesota (copyright 2004,2005).  You are free to distribute this source
  as long as this notice remains in tact.  End users of this code agree to
  take responsibility for how this code is used and John C. Martin and the
  University of Minnesota bear no liability.*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "mkvector.h"
#define FLIP(x,y) {temp=(x);(x)=(y);(y)=temp;}

void gaussjordan(double **a, int nc, double *b)
{
  /*A linear solution by the Gauss-Jordan method of linear elimination with 
    pivoting.  This routine is useful for the figuring the solution of 
    differential equations.  a[1..nc][1..nc] and b[1..nc]*/
  int *colindex,*rowindex,*pivot;
  int i,icol,irow,j,k,l,m;
  double mx,dum,invrs,temp;

  colindex=ivector1d(1,nc);
  rowindex=ivector1d(1,nc);;
  pivot=ivector1d(1,nc);
  for (j=1;j<=nc;j++) pivot[j]=0;
  for (i=1;i<=nc;i++) {
    mx=0.0;
    for (j=1;j<=nc;j++)
      if (pivot[j] !=1)
	for (k=1;k<=nc;k++) {
	  if (pivot[k]==0) {
	    if (fabs(a[j][k])>= mx) {
	      mx=fabs(a[j][k]);
	      irow=j;
	      icol=k;
	    }
	  } else if (pivot[k] >1 ) {
	    fprintf(stderr,"GaussJordan: Singular Matrix\n");
	    exit(1);
	  }
	}
    ++(pivot[icol]);
    if (irow!=icol) {
	for (l=1;l<=nc;l++) FLIP(a[irow][l],a[icol][l])
        FLIP(b[irow],b[icol])
    }
    if (i>nc) printf("BIG Trouble i=%i\n",i);
    rowindex[i]=irow;
    colindex[i]=icol;
    if (a[icol][icol]==0.0) {
      fprintf(stderr,"GaussJordan: Singular Matrix (element %i,%i is zero!)\n",
	      icol,icol);
      exit(2);
    }
    invrs=1.0/a[icol][icol];
    a[icol][icol]=1.0;
    for (l=1;l<=nc;l++) a[icol][l] *= invrs;
    b[icol] *= invrs;
    for (m=1;m<=nc;m++)
      if (m!=icol) {
	dum=a[m][icol];
	a[m][icol]=0.0;
	for (l=1;l<=nc;l++) a[m][l] -= a[icol][l]*dum;
	b[m] -= b[icol]*dum;
      }
  }
  for (l=nc;l>=1;l--) {
    if (rowindex[l] != colindex[l])
      for (k=1;k<=nc;k++)
	FLIP(a[k][rowindex[l]],a[k][colindex[l]]);
  }
  free(colindex);
  free(rowindex);
  free(pivot);

}
