/*The code was written and debugged by John C. Martin at the University of
  Minnesota (copyright 2004,2005).  You are free to distribute this source
  as long as this notice remains in tact.  End users of this code agree to
  take responsibility for how this code is used and John C. Martin and the
  University of Minnesota bear no liability.*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
void fitgaus(double *y, int ndata, double *a);
void MakeXcut(double **apix,long naxes[3],int r1,int r2,int c1,int c2,
              double *cutout);
int xtractor(double **pix,double x1, double x2, double y,
             double top, double base, double *out,int wflag);

int locateghost(double **pix,long naxes[3],
		double hc, double hr,double pa,double d,
		double *gc,double *gr,double *sgc, double *sgr,double *amp)
{
  /*Find the location and dimensions of the ghost
    If gc and gr are entered as zero they are calculated from the
    estimated position of halpha (hc,hr) from the position angle (pa in rad)
    and the distance (d in array pixels).*/
  FILE *fout;
  double a[4],cut[200],y1,y2;
  double cwin=52;
  double rwin=30;
  int debug=0;
  int igc,igr,i;

  if (*gc==0) {
    *gc=(105.*cos(pa)+hc)+.5;
    *gr=(105.*sin(pa)+hr)+.5;
  }
  igc=*gc+.5;
  igr=*gr+.5;
  if (debug) printf("ghost center %i,%i\n",igc,igr);
  //Make row cut
  MakeXcut(pix,naxes,igr-rwin,igr+rwin,igc-10,igc+10,cut);
  y1=0;for (i=0;i<5;i++) y1+=cut[i];y1=y1/5.;
  y2=0;for (i=2*rwin-4;i<2*rwin+1;i++) y2+=cut[i];y2=y2/5.;
  y2=(y2-y1)/(2*rwin-5);
  if (debug) printf("Row cut slope=%g\n",y2);
  for (i=0;i<2*rwin+1;i++) {cut[i]-=y1+y2*(i-2.5);}
  a[1]=cut[(int)rwin+1];
  a[2]=rwin+1;
  if (*sgr==0) a[3]=15; else a[3]=*sgr;
  //Fit Row Cut
  fitgaus(cut,2*rwin+1,a);
  if (debug) {
    printf("Row fit: %g %g %g\n",a[1],a[2],a[3]);
    fout=fopen("rowcut.txt","w");
    for (i=0;i<rwin*2+1;i++) fprintf(fout,"%i %g\n",i-(int)rwin,cut[i]);
    fclose(fout);
    fout=fopen("rowfit.txt","w");
    for (i=0;i<rwin*2+1;i++) {
      y1=a[1]*exp(-pow((i-a[2])/a[3],2));
      fprintf(fout,"%i %g\n",i-(int)rwin,y1);
    }
    fclose(fout);
  }
  *gr=a[2]-rwin+igr;
  *sgr=a[3];
  //Make Collumn cut
  xtractor(pix,(double)(igc-cwin),(double)igc+cwin,(double)igr,14,16,cut,1);
  y1=0;for (i=0;i<5;i++) y1+=cut[i];y1=y1/5.;
  y2=0;for (i=2*cwin-4;i<2*cwin+1;i++) y2+=cut[i];y2=y2/5.;
  y2=(y2-y1)/(2*cwin-5);
  if (debug) printf("Col cut slope=%g\n",y2);
  for (i=0;i<2*cwin+1;i++) {cut[i]-=y1+y2*(i-2.5);}
  a[1]=cut[(int)cwin+1];
  a[2]=cwin+1.;
  if (*sgc==0) a[3]=30; else a[3]=*sgc;
  //Fit Collumn cut
  fitgaus(cut,2*cwin+1,a);
  if (debug) {
    printf("Col fit: %g %g %g\n",a[1],a[2],a[3]);
    fout=fopen("colcut.txt","w");
    for (i=0;i<2*cwin+1;i++) fprintf(fout,"%i %g\n",i-(int)cwin,cut[i]);
    fclose(fout);
    fout=fopen("colfit.txt","w");
    for (i=0;i<2*cwin+1;i++) {
      y1=a[1]*exp(-pow((i-a[2])/a[3],2));
      fprintf(fout,"%i %g\n",i-(int)cwin,y1);
    }
    fclose(fout);
  }
  *gc=a[2]-cwin+igc;
  *sgc=a[3];
  *amp=a[1];
  
  return(0);
}
