/*The code was written and debugged by John C. Martin at the University of
  Minnesota (copyright 2004,2005).  You are free to distribute this source
  as long as this notice remains in tact.  End users of this code agree to
  take responsibility for how this code is used and John C. Martin and the
  University of Minnesota bear no liability.*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int rmghost(double **pix,int flag,double amp,double r0,
	    double c0,double sgc,double sgr)
{
  /*Remove the Halpha ghost 
     Parameters for the 2D Gaussian:
      amp = peak amplitude of distribution
      c0  = column of center
      r0  = row of center
      sgc = "sigma" in x direction (columns)
      sgr = "sigma" in y direction (columns)
     Flag settings:
      flag = 0  => subtract the modeled ghost from pix array
      flag = 1  => return the modeled ghost in the pix array
     Return:
      0 if everything okay
      1 if there is an error
  */
  int r,c,r1,r2,c1,c2;
  double x,y;
  //int debug=1;

  //Set Limits
  r0-=1;//correct for first row being 0 not 1
  c0-=1;//correct for first column being 0 not 1
  r1=r0-4*sgr;
  r2=r0+4*sgr;
  c1=c0-4*sgc;
  c2=c0+4*sgc;
  //Do it
  for (c=c1;c<=c2;c++)
    for (r=r1;r<=r2;r++) {
      x=exp(-pow((c-c0)/sgc,2));
      y=exp(-pow((r-r0)/sgr,2));
      switch (flag){
      case 0: {pix[r][c]-=amp*x*y;break;}
      case 1: {pix[r][c]=amp*x*y;break;}
      default: {
	fprintf(stderr,"RMGHOST: Unknown flag %i.... aborting\n",flag);
	return(1);
      }
      }
    }

  return(0);
}
 
