/*The code was written and debugged by John C. Martin at the University of
  Minnesota (copyright 2004,2005).  You are free to distribute this source
  as long as this notice remains in tact.  End users of this code agree to
  take responsibility for how this code is used and John C. Martin and the
  University of Minnesota bear no liability.*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

int xtractor(double **pix,double x1, double x2, double y, 
	     double top, double base, double *out,int wflag)
{
  /*xtract from column x1 to x2 centered on row y with mesa fuction using 
    parameters top & base*/

  int r,c,r1,dr,ix1,ix2,nel;
  double *w,sum,slp,t1,t2,b1,b2,srt,end,wsum;
  int debug=0;

  if (debug) printf("Start xtractor\nExtract from %g %g along row %g with mesa %g x %g\n",x1,x2,y,top,base);
  t1=y-top/2.;
  t2=y+top/2.;
  b1=y-base/2.;
  b2=y+base/2;
  r1=(int)b1;
  dr=(int)(b2+.5)-r1;
  slp=1./(base/2.-top/2.);
  w=malloc(sizeof(double)*(dr+1));
  wsum=0;
  if (debug) printf("Rows %i to %i\nWeights :",r1,r1+dr);
  for (r=0;r<=dr;r++) {
    w[r]=0;
    if (debug>1) printf("%g %g %g %g %g %g\n",r+r1-.5,r+r1+.5,b1,t1,t2,b2);
    /*does the pixel fall in region 1?*/
    if (((b1>=r+r1-.5)&&(b1<r+r1+.5))||((t1>=r+r1-.5)&&(t1<r+r1+.5))||((b1<r+r1-.5)&&(t1>r+r1+.5))) {
      if (debug>1) printf("In 1\n");
      if (r+r1-.5<b1) srt=b1;
      else srt=r+r1-.5;
      if (r+r1+.5>t1) end=t1;
      else end=r+r1+.5;
      w[r]+=slp*(end*end/2.-srt*srt/2.+b1*(srt-end));
    }
    /*does the pixel fall in region 2?*/
    if (((t1>=r+r1-.5)&&(t1<r+r1+.5))||((t2>=r+r1-.5)&&(t2<r+r1+.5))||((t1<r+r1-.5)&&(t2>r+r1+.5))) {
      if (debug>1) printf("In 2\n");
      if (r+r1-.5<t1) srt=t1;
      else srt=r+r1-.5;
      if (r+r1+.5>t2) end=t2;
      else end=r+r1+.5;
      w[r]+=end-srt;
    }
    /*does the pixel fall in region 3?*/
    if (((t2>=r+r1-.5)&&(t2<r+r1+.5))||((b2>=r+r1-.5)&&(b2<r+r1+.5))||((t2<r+r1-.5)&&(b2>r+r1+.5))) {
      if (debug>1) printf("In 3\n");
      if (r+r1-.5<t2) srt=t2;
      else srt=r+r1-.5;
      if (r+r1+.5>b2) end=b2;
      else end=r+r1+.5;
      w[r]+=end-end*end*slp/2.-srt+slp*srt*srt/2.+end*slp*t2-slp*srt*t2; 
    }
    wsum+=w[r];
    if (debug) printf(" %g",w[r]);
  }
  if (debug) printf("\n");

  ix1=(int)(x1+.5)+1;//+1 corrects for first row =0
  ix2=(int)(x2+.5)+1;
  nel=0;
  for (c=ix1;c<=ix2;c++) {
    sum=0;
    for (r=r1;r<=r1+dr;r++) {
      sum+=pix[r][c]*w[r-r1];
    }
    out[c-ix1]=sum;//wsum;
    if (wflag) out[c-ix1]=out[c-ix1]/wsum;    
// if you divide by wsum here it reverts to per pixel
//   if you want per pixel out do not divide by wsum
    nel++;
  }
  return(nel);
}
